searchData={"items":[{"type":"module","title":"Sofa","doc":"Documentation for `Sofa`, a test-driven idiomatic Apache CouchDB client.\n\n> If the only tool you have is CouchDB, then\n> everything looks like `{:ok, :relax}`","ref":"Sofa.html"},{"type":"module","title":"Examples - Sofa","doc":"iex> Sofa.init() |> Sofa.client() |> Sofa.connect!()\n\n    %{\n      \"couchdb\" => \"Welcome\",\n      \"features\" => [\"access-ready\", \"partitioned\", \"pluggable-storage-engines\",\n        \"reshard\", \"scheduler\"],\n      \"git_sha\" => \"ce596c65d\",\n      \"uuid\" => \"59c032d3a6adcd5b44315137a124bf69\",\n      \"vendor\" => %{\"name\" => \"FreeBSD\"},\n      \"version\" => \"3.1.1\"\n    }","ref":"Sofa.html#module-examples"},{"type":"function","title":"Sofa.active_tasks/1","doc":"Get _active_tasks. Only available to admin users.","ref":"Sofa.html#active_tasks/1"},{"type":"function","title":"Sofa.all_dbs/1","doc":"List all databases. Only available to admin users.","ref":"Sofa.html#all_dbs/1"},{"type":"function","title":"Sofa.auth_info/1","doc":"Returns user & password credentials extracted from a typical %URI{} userinfo\nfield, as a Tesla-compatible authorization header. Currently only supports\nBasicAuth user:password combination.","ref":"Sofa.html#auth_info/1"},{"type":"function","title":"Examples - Sofa.auth_info/1","doc":"iex> Sofa.auth_info(\"admin:password\")\n    %{username: \"admin\", password: \"password\"}\n\n    iex> Sofa.auth_info(\"blank:\")\n    %{username: \"blank\", password: \"\"}\n\n    iex> Sofa.auth_info(\"garbage\")\n    %{}","ref":"Sofa.html#auth_info/1-examples"},{"type":"function","title":"Sofa.client/1","doc":"Builds Telsa runtime client, with appropriate middleware header credentials,\nfrom supplied %Sofa{} struct.","ref":"Sofa.html#client/1"},{"type":"function","title":"Sofa.connect/1","doc":"Given an existing %Sofa{} struct, or a prepared URI, attempts to connect\nto the CouchDB instance, and returns an updated %Sofa{} to use in future\nconnections to this server, using the same HTTP credentials.\n\nReturns an updated `{:ok, %Sofa{}}` on success, or `{:error, reason}`, if\nfor example, the URL is unreachable, times out, supplied credentials are\nrejected by CouchDB, or returns unexpected HTTP status codes.","ref":"Sofa.html#connect/1"},{"type":"function","title":"Sofa.connect!/1","doc":"Bang! wrapper around Sofa.connect/1; raises exceptions on error.","ref":"Sofa.html#connect!/1"},{"type":"function","title":"Sofa.init/1","doc":"Takes an optional parameter, the CouchDB uri, and returns a struct\ncontaining the usual CouchDB server properties. The URI may be given\nas a string or as a %URI struct.\n\nThis should be piped into `Sofa.client/1` to create the HTTP client,\nwhich is stored inside the struct with correct authentication information.","ref":"Sofa.html#init/1"},{"type":"function","title":"Examples - Sofa.init/1","doc":"iex> Sofa.init(\"https://very:Secure@foreignho.st:6984/\")\n\n    %Sofa{\n      auth: \"very:Secure\",\n      features: nil,\n      uri: %URI{\n        authority: \"very:Secure@foreignho.st:6984\",\n        fragment: nil,\n        host: \"foreignho.st\",\n        path: \"/\",\n        port: 6984,\n        query: nil,\n        scheme: \"https\",\n        userinfo: \"very:Secure\"\n      },\n      uuid: nil,\n      vendor: nil,\n      version: nil\n    }","ref":"Sofa.html#init/1-examples"},{"type":"function","title":"Sofa.raw/6","doc":"Minimal wrapper around native CouchDB HTTP API, allowing an escape hatch\nfor raw functionality, and as the core abstraction layer for Sofa itself.","ref":"Sofa.html#raw/6"},{"type":"function","title":"Sofa.raw!/5","doc":"Bang! wrapper around Sofa.raw/1; raises exceptions on error.","ref":"Sofa.html#raw!/5"},{"type":"type","title":"Sofa.t/0","doc":"","ref":"Sofa.html#t:t/0"},{"type":"module","title":"Sofa.Cushion","doc":"Internal Helpers for Sofa, with a vanity naming convention.\n\n> If the only tool you have is CouchDB, everything is very\n> uncomfortable without a Cushion.","ref":"Sofa.Cushion.html"},{"type":"function","title":"Sofa.Cushion.untaint_headers/1","doc":"Sanitise HTTP headers into ones we trust and format, and drop the rest.\nThis is necessary because proxies, clients, HTTP1* and HTTP2 all disagree\nabout whether headers should be upper, lower, camel, snake, or wtf case.\n\n    Server : CouchDB/3.1.1 (Erlang OTP/22)\n    X-Couch-Request-Id : f5b74b7038\n    X-Couchdb-Body-Time : 0\n    Cache-Control : must-revalidate\n    Content-Length : 443\n    Content-Type : application/json\n    Date : Sun, 25 Apr 2021 18:43:36 GMT\n    Etag : \"4-322add00c33cab838bf9d7909f18d4f5\"","ref":"Sofa.Cushion.html#untaint_headers/1"},{"type":"module","title":"Sofa.DB","doc":"Documentation for `Sofa.DB`, a test-driven idiomatic Apache CouchDB client.\n\n> If the only tool you have is CouchDB, then\n> everything looks like `{:ok, :relax}`","ref":"Sofa.DB.html"},{"type":"module","title":"Examples - Sofa.DB","doc":"iex>  sofa = Sofa.init(\"http://admin:passwd@localhost:5984/\")\n          |> Sofa.client()\n          |> Sofa.connect!()\n        #Sofa<\"...\">\n\n    iex> Sofa.DB.create(sofa, \"testy\")\n        {:ok,\n        #Sofa ,\n        %Sofa.Response{\n          body: %{\"ok\" => true},\n          headers: %{\n            cache_control: \"must-revalidate\",\n            content_length: 95,\n            content_type: \"application/json\",\n            couch_body_time: 0,\n            couch_request_id: \"aa6cc50741\",\n            date: \"Sun, 25 Apr 2021 20:04:34 GMT\",\n            server: \"CouchDB/3.1.1 (Erlang OTP/22)\"\n          },\n          method: :put,\n          query: [],\n          status: 201,\n          url: \"http://localhost:5984/testy7\"\n        }}\n\n    iex> Sofa.DB.open!(sofa, \"testy\")\n      #Sofa","ref":"Sofa.DB.html#module-examples"},{"type":"function","title":"Sofa.DB.create/2","doc":"create DB. Only available to cluster admin users.","ref":"Sofa.DB.html#create/2"},{"type":"function","title":"Sofa.DB.delete/2","doc":"Delete DB. Only available to cluster admin users.","ref":"Sofa.DB.html#delete/2"},{"type":"function","title":"Sofa.DB.info/2","doc":"Get DB info. Only available to database & cluster admin users.","ref":"Sofa.DB.html#info/2"},{"type":"function","title":"Sofa.DB.open/2","doc":"Open DB. Checks if supplied credentials have access to the DB, and returns\nupdated `%Sofa{}` struct with DB. Ideal for subsequent use to read/write Docs.","ref":"Sofa.DB.html#open/2"},{"type":"function","title":"Sofa.DB.open!/2","doc":"Bang! version of open/2. Opens DB and raises on failure. Ideal for piping\ndirectly into reading and writing Docs.","ref":"Sofa.DB.html#open!/2"},{"type":"module","title":"Sofa.Doc","doc":"Documentation for `Sofa.Doc`, a test-driven idiomatic Apache CouchDB client.\n\n> If the only tool you have is CouchDB, then\n> everything looks like `{:ok, :relax}`","ref":"Sofa.Doc.html"},{"type":"module","title":"Examples - Sofa.Doc","doc":"iex> Sofa.Doc.new()","ref":"Sofa.Doc.html#module-examples"},{"type":"function","title":"Sofa.Doc.coerce_to_elixir_type/1","doc":"Coerces a CouchDB \"type\" field to an existing atom. It is assumed that there\nwill be a related Elixir Module Type of the same name. Elixir prefixes Module\nnames with Elixir. and then elides this in iex, tests, and elsewhere, but\nhere we need to make that explicit.\n\nThe \"user\" type is special-cased as it is already present in CouchDB /_users\ndatabase.\n\nThis function is expected to be paired up with Ecto Schemas to properly manage\nthe appropriate fields in your document body.","ref":"Sofa.Doc.html#coerce_to_elixir_type/1"},{"type":"function","title":"Sofa.Doc.delete/2","doc":"delete doc","ref":"Sofa.Doc.html#delete/2"},{"type":"function","title":"Sofa.Doc.exists/2","doc":"Check if doc exists via `HEAD /:db/:doc` and returns either:\n\n- `{:error, not_found}` # doc doesn't exist\n- `{:ok, %Sofa.Doc{}}` # doc exists and has metadata","ref":"Sofa.Doc.html#exists/2"},{"type":"function","title":"Sofa.Doc.exists?/2","doc":"Check if doc exists via `HEAD /:db/:doc` and returns either true or false","ref":"Sofa.Doc.html#exists?/2"},{"type":"function","title":"Sofa.Doc.from_map/1","doc":"Converts CouchDB-native JSON-friendly map to internal `%Sofa.Doc{}` format","ref":"Sofa.Doc.html#from_map/1"},{"type":"function","title":"Examples - Sofa.Doc.from_map/1","doc":"iex> %{ \"_id\" => \"smol\", \"_rev\" => \"1-cute\", \"yes\" => true}\n          |> from_map()\n\n      %Sofa.Doc{\n        attachments: nil,\n        body: %{\"yes\" => true},\n        id: \"smol\",\n        rev: \"1-cute\",\n        type: nil\n      }","ref":"Sofa.Doc.html#from_map/1-examples"},{"type":"function","title":"Sofa.Doc.get/2","doc":"GET doc and returns standard HTTP status, or the requested doc\n\n- {:error, :not_found}  # doc doesn't exist, or similar HTTP status\n- %Sofa.Doc{}           # doc exists and has metadata","ref":"Sofa.Doc.html#get/2"},{"type":"function","title":"Sofa.Doc.new/1","doc":"Creates a new (empty) document","ref":"Sofa.Doc.html#new/1"},{"type":"function","title":"Sofa.Doc.put/2","doc":"Optimistically write/update doc assuming rev matches","ref":"Sofa.Doc.html#put/2"},{"type":"type","title":"Sofa.Doc.t/0","doc":"","ref":"Sofa.Doc.html#t:t/0"},{"type":"function","title":"Sofa.Doc.to_map/1","doc":"Converts internal `%Sofa.Doc{}` format to CouchDB-native JSON-friendly map","ref":"Sofa.Doc.html#to_map/1"},{"type":"function","title":"Examples - Sofa.Doc.to_map/1","doc":"iex> %Sofa.Doc{id: \"smol\", rev: \"1-cute\", body: %{\"yes\" => true}}\n        |> to_map()\n\n      %{ \"_id\" => \"smol\", \"_rev\" => \"1-cute\", \"yes\" => true}","ref":"Sofa.Doc.html#to_map/1-examples"},{"type":"module","title":"Sofa.Response","doc":"HTTP Response Handler for Sofa, the Elixir-native Apache CouchDB client\n\n> If the only tool you have is CouchDB, then\n> everything looks like `{:ok, :relax}`","ref":"Sofa.Response.html"},{"type":"module","title":"Examples - Sofa.Response","doc":"iex> Sofa.init() |> Sofa.client() |> Sofa.connect!() |> Sofa.raw(\"/_up\")\n    %Sofa.Response{\n      body: nil,\n      headers: %{\n        cache_control: \"must-revalidate\",\n        content_length: 95,\n        content_type: \"application/json\",\n        couch_body_time: 0,\n        couch_request_id: \"aa6cc50741\",\n        date: \"Sun, 25 Apr 2021 20:04:34 GMT\",\n        server: \"CouchDB/3.1.1 (Erlang OTP/22)\"\n      },\n      method: :get,\n      query: [],\n      status: 200,\n      url: \"http://localhost:5984/_up\"\n    }","ref":"Sofa.Response.html#module-examples"},{"type":"type","title":"Sofa.Response.t/0","doc":"","ref":"Sofa.Response.html#t:t/0"},{"type":"module","title":"Sofa.User","doc":"Documentation for `Sofa.User`, a test-driven idiomatic Apache CouchDB client.\n\n> If the only tool you have is CouchDB, then\n> everything looks like `{:ok, :relax}`\n\nThe User module provides simple wrappers around the usual Sofa.DB and\nSofa.Doc functions, specifically for the `_users` DB.\n\nYou need appropriate permissions for this to work - either as administrator,\nor alternatively, if your user/group have permissions *and* your `local.ini`\nhas `users_security_editable = true` set in `[couchdb]` section. If you\nuse a group to assign permissions, `users` is a good choice.","ref":"Sofa.User.html"},{"type":"function","title":"Sofa.User.generate_random_secret/1","doc":"Generate small, random secret of length 16-64 characters inclusive,\nusing base64 encoding, suitable for CouchDB default user passwords.","ref":"Sofa.User.html#generate_random_secret/1"},{"type":"function","title":"Examples - Sofa.User.generate_random_secret/1","doc":"> Sofa.User.generate_random_secret()\n    \"E3gZtvNhF7pkHpeyoPjMnfRJidI5nRHD/MeRPR11jMKBxcMUXl75U8msnRj1bG/R\"","ref":"Sofa.User.html#generate_random_secret/1-examples"},{"type":"function","title":"Sofa.User.get/2","doc":"GET doc and returns standard HTTP status codes, or the user doc\n\n- `{:error, :not_found}` - doc doesn't exist\n- `%Sofa.Doc{}` - doc exists and has metadata","ref":"Sofa.User.html#get/2"},{"type":"function","title":"Sofa.User.new/3","doc":"Create a new _user Sofa.Doc with the usual attributes per\n\n  # https://docs.couchdb.org/en/stable/intro/security.html?highlight=_users#creating-a-new-user\n\n    { _id: \"org.couchdb.user:jan\",\n      \"name\": \"jan\",\n      \"password\": \"apple\",\n      \"roles\": [\"chair\"],\n      \"type\": \"user\"\n    }","ref":"Sofa.User.html#new/3"},{"type":"function","title":"Examples - Sofa.User.new/3","doc":"iex> Sofa.User.new(\"jan\", \"apple\", [\"pointy_hat\", \"users\"])\n\n    %Sofa.Doc{\n      attachments: %{},\n      body: %{\"name\" => \"jan\", \"password\" => \"apple\", \"roles\" => [\"pointy_hat\", \"users\"]},\n      id: \"org.couchdb.user:jan\",\n      rev: nil,\n      type: :user\n    }","ref":"Sofa.User.html#new/3-examples"},{"type":"function","title":"Sofa.User.put/2","doc":"PUT a valid Sofa.Doc with user attributes into the `/_users` DB.\n\nIf `doc.body.password` is supplied, remove hash info and update password.","ref":"Sofa.User.html#put/2"},{"type":"function","title":"Sofa.User.reset_password/2","doc":"Resets user password. NB you still need to write this doc to CouchDB.","ref":"Sofa.User.html#reset_password/2"},{"type":"exception","title":"Sofa.Error","doc":"","ref":"Sofa.Error.html"},{"type":"extras","title":"Sofa","doc":"# Sofa\n\nA straightforwards, idiomatic [CouchDB] client.\n\nThe intention is to provide an idiomatic Elixir client, that can play\nnicely with Ecto, Maps, and in particular, Structs and Protocols. You\nshould be able to store a Struct in CouchDB, and have it come back to\nyou as a Struct again, assuming you're not doing anything too messy,\nsuch as nested structs, or trying to store pids, refs, and other\ndistinctly non-JSON things.","ref":"readme.html"},{"type":"extras","title":"Installation - Sofa","doc":"It is recommended to use a `Tesla.Adapter`. While in principle these are\nall equivalent, in practice, their patterns for handling query\nparameters, headers, empty HTTP bodies, IPv6, and generally dealing with\n`nil`, `true`, `false` and so forth mean that they are not created\nequal. This library should work, in most cases transparently, and if\nnot, we welcome tests and converters to address any shortcomings.\n\nSofa makes no guarantees about specific HTTP modules, but should run\nwith:\n\n- default Erlang `httpc` \"no dependencies!\"\n- Mint and Finch\n\nThe package can be installed by adding `sofa` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:sofa, \"~> 0.1.0\"}\n  ]\nend\n```\n\n```elixir\n# config/config.exs\nimport Config\n\nif config_env() == :test do\n  config :tesla, adapter: Tesla.Mock\nelse\n  config :tesla, adapter: Tesla.Adapter.Mint\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Docs - Sofa","doc":"- [hexdocs] as usual has all the goodies\n- the [CouchDB API] should map very closely to Sofa\n\nSofa really only has 2 important abstractions that live above the\nCouchDB API:\n\n- `%Sofa{}` aka `Sofa.t()` which is a struct that wraps your HTTP API\n    connection, along with any custom headers & settings you may\n    require, and the returned data from the CouchDB server you connect\n    to, including feature flags and vendor settings. As a convenience,\n    it also doubles as your \"database\" struct, as that's really only a\n    single additional field to be inserted into the CouchDB URL\n- `%Sofa.Doc{}` aka `Sofa.Doc.t()` which is the main struct you'll work\n    with. We've tried to keep it as close to the [CouchDB API] as possible,\n    so aside from `id`, `rev`, and the `attachments` stubs, all the\n    JSON is contained in a `body` and Sofa keeps out of your way.","ref":"readme.html#docs"},{"type":"extras","title":"Road Map - Sofa","doc":"While not yet implemented, Sofa wants to support \"native\" Elixir struct\nusage, where you implement the Protocol to convert your custom Struct\nto/from Sofa, and Sofa will use the `type` key that is commonly used in\nCouchDB to detect & marshall your Struct directly to/from CouchDB's JSON\nAPI transparently.\n\n- [x] server:   `Sofa.*`\n- [x] raw HTTP: `Sofa.Raw.*`\n- [x] database: `Sofa.DB.*`\n- [x] document: `Sofa.Doc.*`\n- [x] user:     `Sofa.User.*`\n- [ ] attachments\n- [ ] transparent Struct API\n- [ ] view:     `Sofa.View.*`\n- [ ] changes:  `Sofa.Changes.*`\n- [ ] timeouts for requests and inactivity\n- [ ] bearer token authorisation\n- [ ] runtime tracing filterable by method & URL\n- [ ] embeddable within CouchDB BEAM runtime\n- [ ] native CouchDB erlang term support","ref":"readme.html#road-map"},{"type":"extras","title":"Usage - Sofa","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Connecting to CouchDB - Sofa","doc":"`Sofa.init/1` and `Sofa.client/1` are effectively static structures, so\nyou can build them at compile time, or store them efficiently in ETS\ntables, or `persistent_term` for faster access.\n\n`Sofa.connect!/1` needs access to the CouchDB server, to verify that\nyour credentials are sufficient, and to retrieve feature flags and\nvendor settings.\n\n> Exactly how you use this, is dependent on your `Tesla.Adapter` and\n> supervision trees. Make sure that you're not opening a new TCP\n> connection for every call to the database, and then leave them\n> dangling until your app or the server runs of of connections!\n\nThe `Sofa.DB.open!/2` call also does similar checks, ensuring you have\nat least permissions to access the database, in some form. There is\nnothing that changes over time within this struct, so feel free to cache\nit \"for a while\" in your processes if that helps.\n\n```elixir\n# connect to CouchDB and ensure our credentials are valid\niex> sofa = Sofa.init(\"http://admin:passwd@localhost:5984/\")\n        |> Sofa.client()\n        |> Sofa.connect!()\n    #Sofa  db = Sofa.DB.open!(\"mydb\")\n    #Sofa \n```","ref":"readme.html#connecting-to-couchdb"},{"type":"extras","title":"Doc Usage - Sofa","doc":"There shouldn't be any surprises here - an Elixir `Map %{}` becomes the\n`body` of the `%Sofa.Doc{}` struct, and the usual CouchDB internal\nfields are available as additional atom fields off the struct:\n\n```elixir\niex>  doc = %{\"_id\" => \"smol\", \"cute\" => true} |> Sofa.Doc.from_map()\n    %Sofa.Doc{\n    attachments: nil,\n    body: %{\n        \"cute\" => true\n    },\n    id: \"smol\",\n    rev: nil,\n    type: nil\n    }\niex> doc |> Sofa.Doc.to_map()\n    %{\n        \"_id\" => \"smol\",\n        \"cute\" => true\n    }\n# fetch and retrieve documents works like you'd expect\niex> Sofa.Doc.exists?(db,\"missing\")\n    false\n```","ref":"readme.html#doc-usage"},{"type":"extras","title":"Raw Mode - Sofa","doc":"Sometimes you just want to re-upholster the Couch yourself. That's fine,\nraw mode is here to help you:\n\n```elixir\n# raw mode gives you direct access to CouchDB API, with JSONification\niex> db = Sofa.init(\"http://admin:passwd@localhost:5984/\")\n        |> Sofa.client()\n        |> Sofa.connect!()\n        |> Sofa.raw(\"/_membership\")\n{:ok,\n #Sofa ,\n %Sofa.Response{\n   body: %{\n     \"all_nodes\" => [\"couchdb@127.0.0.1\"],\n     \"cluster_nodes\" => [\"couchdb@127.0.0.1\"]\n   },\n   headers: %{\n     cache_control: \"must-revalidate\",\n     content_length: 74,\n     content_type: \"application/json\",\n     date: \"Wed, 28 Apr 2021 14:11:10 GMT\",\n     server: \"CouchDB/3.2.0 (Erlang OTP/22)\"\n   },\n   method: :get,\n   query: [],\n   status: 200,\n   url: \"http://localhost:5984/_membership\"\n }}\n```","ref":"readme.html#raw-mode"},{"type":"extras","title":"Contributing - Sofa","doc":"If raw mode can't do it, send a PR, and we'll `make it so`. If you find\nyourself reaching for raw mode often, consider a PR that extends Sofa\nitself?\n\nSofa should pass credo, and also respect dialyzer, via `make lint`.","ref":"readme.html#contributing"},{"type":"extras","title":"Thanks - Sofa","doc":"To the CouchDB team, a part of my life for more than a decade. Relax.\n\n[hex]: https://hex.pm/packages/sofa\n[CouchDB]: https://couchdb.apache.org/\n[hexdocs]: https://hexdocs.pm/sofa\n[CouchDB API]: https://docs.couchdb.org/","ref":"readme.html#thanks"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.3 - Changelog","doc":"- Fix module doc `iex>` DocTest indentation to make HexDocs look pretty","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.2 - Changelog","doc":"- Initial release","ref":"changelog.html#0-1-2"},{"type":"extras","title":"LICENSE","doc":"BSD 2-Clause License\n\nCopyright (c) 2021-2038 SkunkWerks, GmbH\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n1. Redistributions of source code must retain the above copyright notice, this\n   list of conditions and the following disclaimer.\n\n2. Redistributions in binary form must reproduce the above copyright notice,\n   this list of conditions and the following disclaimer in the documentation\n   and/or other materials provided with the distribution.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}